@@ -1,11 +1,12 @@
-#' DTW calculation guided by Lemire's lower bounds
+#' DTW distance matrix guided by Lemire's improved lower bound
 #'
-#' Calculation of a distance matrix with the Dynamic Time Warping (DTW) distance guided Lemire's
+#' Calculation of a distance matrix with the Dynamic Time Warping (DTW) distance guided by Lemire's
 #' improved lower bound (LB_Improved).
 #'
 #' This function first calculates an initial estimate of a distance matrix between two sets of time series
 #' using LB_Improved. Afterwards, it uses the estimate to calculate the corresponding true DTW distance
-#' between \emph{only} the nearest neighbors of each series in \code{x} found in \code{y}.
+#' between \emph{only} the nearest neighbors of each series in \code{x} found in \code{y}, and it continues
+#' iteratively until no changes in the nearest neighbors occur.
 #'
 #' If only \code{x} is provided, the distance matrix is calculated between all its time series.
 #'
@@ -30,19 +31,16 @@
 #'
 #' This function uses a lower bound that is only defined for time series of equal length.
 #'
-#' If you use the version registered with \code{proxy}, and because of how \code{\link[proxy]{dist}}
-#' works, the latter's \code{pairwise} argument will not work with this distance. You can use the custom argument
-#' \code{force.pairwise} to get the correct result (which is, effectively, calculating only DTW distances).
-#'
 #' @seealso
 #'
 #' \code{\link{lb_keogh}}, \code{\link{lb_improved}}
 #'
 #' @references
 #'
-#' Lemire D (2009). ``Faster retrieval with a two-pass dynamic-time-warping lower bound .'' \emph{Pattern Recognition}, \strong{42}(9), pp.
-#' 2169 - 2180. ISSN 0031-3203, \url{http://dx.doi.org/10.1016/j.patcog.2008.11.030}, \url{
-#' http://www.sciencedirect.com/science/article/pii/S0031320308004925}.
+#' Lemire D (2009). ``Faster retrieval with a two-pass dynamic-time-warping lower bound .''
+#' \emph{Pattern Recognition}, \strong{42}(9), pp. 2169 - 2180. ISSN 0031-3203,
+#' \url{http://dx.doi.org/10.1016/j.patcog.2008.11.030},
+#' \url{http://www.sciencedirect.com/science/article/pii/S0031320308004925}.
 #'
 #' @examples
 #'
@@ -96,24 +94,26 @@
 #'
 #' @param x,y A matrix where rows are time series, or a list of time series.
 #' @param window.size Window size to use with the LB and DTW calculation. See details.
-#' @param norm Pointwise distance. Either \code{L1} for Manhattan distance or \code{L2} for Euclidean.
+#' @param norm Pointwise distance. Either \code{"L1"} for Manhattan distance or \code{"L2"} for Euclidean.
 #' @param error.check Should inconsistencies in the data be checked?
-#' @param force.pairwise Calculate pairwise distances. See the notes.
+#' @param pairwise Calculate pairwise distances?
 #'
 #' @return The distance matrix with class \code{crossdist}.
 #'
 #' @export
 
 dtw_lb <- function(x, y = NULL, window.size = NULL, norm = "L1",
-                   error.check = TRUE, force.pairwise = FALSE) {
+                   error.check = TRUE, pairwise = FALSE) {
 
      norm <- match.arg(norm, c("L1", "L2"))
 
+     method <- ifelse(norm == "L1", "DTW", "DTW2")
+
      X <- consistency_check(x, "tsmat")
 
      check_parallel()
 
-     if (force.pairwise) {
+     if (pairwise) {
           if (is.null(y))
                Y <- x
           else
@@ -134,49 +134,40 @@ dtw_lb <- function(x, y = NULL, window.size = NULL, norm = "L1",
                        .combine = c,
                        .multicombine = TRUE,
                        .packages = "dtwclust") %dopar% {
-                            switch(EXPR = norm,
-
-                                   L1 = proxy::dist(X, Y,
-                                                    pairwise = TRUE,
-                                                    method = "DTW",
-                                                    dist.method = "L1",
-                                                    window.type = window.type,
-                                                    window.size = window.size),
-
-                                   L2 = proxy::dist(X, Y,
-                                                    pairwise = TRUE,
-                                                    method = "DTW2",
-                                                    window.type = window.type,
-                                                    window.size = window.size))
+                            proxy::dist(X, Y,
+                                        pairwise = TRUE,
+                                        method = method,
+                                        dist.method = norm,
+                                        window.type = window.type,
+                                        window.size = window.size)
                        }
 
           return(D)
      }
 
-     ## NOTE: I tried starting with LBK estimate, refining with LBI and then DTW, but overall,
+     window.size <- consistency_check(window.size, "window")
+
+     ## NOTE: I tried starting with LBK estimate, refining with LBI and then DTW but, overall,
      ## it was usually slower, almost the whole matrix had to be recomputed for LBI
 
      if (!is.null(y)) {
           Y <- consistency_check(y, "tsmat")
 
-          ## Initial estimate
-          D <- proxy::dist(X, Y, method = "LBI",
-                           window.size = window.size,
-                           norm = norm,
-                           error.check = error.check,
-                           force.symmetry = FALSE)
+          force.symmetry <- FALSE
 
      } else {
           Y <- X
 
-          ## Initial estimate
-          D <- proxy::dist(X, Y, method = "LBI",
-                           window.size = window.size,
-                           norm = norm,
-                           error.check = error.check,
-                           force.symmetry = TRUE)
+          force.symmetry <- TRUE
      }
 
+     ## Initial estimate
+     D <- proxy::dist(X, Y, method = "LBI",
+                      window.size = window.size,
+                      norm = norm,
+                      error.check = error.check,
+                      force.symmetry = force.symmetry)
+
      ## Update with DTW
      new.indNN <- apply(D, 1L, which.min) # index of nearest neighbors
      indNN <- new.indNN + 1L # initialize all different
@@ -187,32 +178,24 @@ dtw_lb <- function(x, y = NULL, window.size = NULL, norm = "L1",
 
           indNew <- split_parallel(indNew)
 
-          exclude <- setdiff(ls(), c("X", "Y", "norm", "indNN", "window.size"))
+          exclude <- setdiff(ls(), c("X", "Y", "norm", "indNN", "window.size", "method"))
 
           dSub <- foreach(indNew = indNew,
                           .combine = c,
                           .multicombine = TRUE,
                           .packages = "dtwclust",
                           .noexport = exclude) %dopar% {
-                               switch(EXPR = norm,
-
-                                      L1 = proxy::dist(X[indNew], Y[indNN[indNew]],
-                                                       pairwise = TRUE,
-                                                       method = "DTW",
-                                                       dist.method = "L1",
-                                                       window.type = "slantedband",
-                                                       window.size = window.size),
-
-                                      L2 = proxy::dist(X[indNew], Y[indNN[indNew]],
-                                                       pairwise = TRUE,
-                                                       method = "DTW2",
-                                                       window.type = "slantedband",
-                                                       window.size = window.size))
+                               proxy::dist(X[indNew], Y[indNN[indNew]],
+                                           pairwise = TRUE,
+                                           method = method,
+                                           dist.method = norm,
+                                           window.type = "slantedband",
+                                           window.size = window.size)
                           }
 
-          D[cbind(1:length(X), indNN)[unlist(indNew), , drop = FALSE]] <- dSub
+          D[cbind(1L:length(X), indNN)[unlist(indNew), , drop = FALSE]] <- dSub
 
-          new.indNN <- apply(D, 1, which.min)
+          new.indNN <- apply(D, 1L, which.min)
      }
 
      attr(D, "method") <- "DTW_LB"
